"""Asynchronous Python client for the easyEnergy API."""

import asyncio
from datetime import date, timedelta

from easyenergy import EasyEnergy, VatOption


async def main() -> None:
    """Show example on fetching the gas prices from easyEnergy."""
    async with EasyEnergy(vat=VatOption.INCLUDE) as client:
        today = date(2024, 1, 30)

        gas_today = await client.gas_prices(start_date=today, end_date=today)
        next_hour = gas_today.utcnow() + timedelta(hours=1)

        print("--- GAS TODAY ---")
        print(f"Extremas prices: {gas_today.extreme_prices}")
        print(f"Average price: {gas_today.average_price}")
        print()
        print(f"Current hourprice: {gas_today.current_price}")
        print(f"Next hourprice: {gas_today.price_at_time(next_hour)}")


if __name__ == "__main__":
    asyncio.run(main())



"""Asynchronous Python client for the easyEnergy API."""

import asyncio
from datetime import date, timedelta

import pytz

from easyenergy import EasyEnergy, VatOption


async def main() -> None:
    """Show example on fetching the energy prices from easyEnergy."""
    async with EasyEnergy(vat=VatOption.INCLUDE) as client:
        local = pytz.timezone("Europe/Amsterdam")
        today = date(2024, 1, 30)
        tomorrow = date(2024, 1, 31)

        # Select your test readings
        switch_e_today: bool = True
        switch_e_tomorrow: bool = True

        if switch_e_today:
            energy_today = await client.energy_prices(start_date=today, end_date=today)
            utc_next_hour = energy_today.utcnow() + timedelta(hours=1)

            print("--- ENERGY TODAY ---")
            print(f"Extremas usage price: {energy_today.extreme_usage_prices}")
            print(f"Extremas return price: {energy_today.extreme_return_prices}")
            print(f"Average usage price: {energy_today.average_usage_price}")
            print(f"Average return price: {energy_today.average_return_price}")
            print(f"Percentage max - Usage: {energy_today.pct_of_max_usage}%")
            print(f"Percentage max - Return: {energy_today.pct_of_max_return}%")
            print()

            highest_time_usage = energy_today.highest_usage_price_time.astimezone(local)
            print(f"Highest price time - Usage: {highest_time_usage}")
            lowest_time_usage = energy_today.lowest_usage_price_time.astimezone(local)
            print(f"Lowest price time - Usage: {lowest_time_usage}")
            print()
            print(f"Current usage price: {energy_today.current_usage_price}")
            print(f"Current return price: {energy_today.current_return_price}")
            print(f"Next hourprice: {energy_today.price_at_time(utc_next_hour)}")

            lower_hours: int = energy_today.hours_priced_equal_or_lower_usage
            highest_hours: int = energy_today.hours_priced_equal_or_higher_return
            print(f"Lower hours (usage): {lower_hours}")
            print(f"Higher hours (return): {highest_hours}")

        if switch_e_tomorrow:
            energy_tomorrow = await client.energy_prices(tomorrow, tomorrow)
            print()
            print("--- ENERGY TOMORROW ---")
            print(f"Extremas usage price: {energy_tomorrow.extreme_usage_prices}")
            print(f"Extremas return price: {energy_tomorrow.extreme_return_prices}")
            print(f"Average usage price: {energy_tomorrow.average_usage_price}")
            print(f"Average return price: {energy_tomorrow.average_return_price}")
            print()
            highest_time = energy_tomorrow.highest_usage_price_time.astimezone(local)
            print(f"Highest price time - Usage: {highest_time}")
            lowest_time = energy_tomorrow.lowest_usage_price_time.astimezone(local)
            print(f"Lowest price time - Usage: {lowest_time}")


if __name__ == "__main__":
    asyncio.run(main())



"""Basic tests for the easyEnergy API."""

# pylint: disable=protected-access
import asyncio
from typing import Any
from unittest.mock import patch

import pytest
from aiodns import DNSResolver
from aiodns.error import DNSError
from aiohttp import ClientError, ClientResponse, ClientSession
from aresponses import Response, ResponsesMockServer

from easyenergy import EasyEnergy
from easyenergy.exceptions import EasyEnergyConnectionError, EasyEnergyError

from . import load_fixtures


async def test_json_request(aresponses: ResponsesMockServer) -> None:
    """Test JSON response is handled correctly."""
    aresponses.add(
        "mijn.easyenergy.com",
        "/nl/api/tariff/test",
        "GET",
        aresponses.Response(
            status=200,
            headers={"Content-Type": "application/json"},
            text=load_fixtures("energy.json"),
        ),
    )
    async with ClientSession() as session:
        client = EasyEnergy(session=session)
        response = await client._request("test")
        assert response is not None
        await client.close()


async def test_internal_session(aresponses: ResponsesMockServer) -> None:
    """Test internal session is handled correctly."""
    aresponses.add(
        "mijn.easyenergy.com",
        "/nl/api/tariff/test",
        "GET",
        aresponses.Response(
            status=200,
            headers={"Content-Type": "application/json"},
            text=load_fixtures("energy.json"),
        ),
    )
    async with EasyEnergy() as client:
        await client._request("test")


async def test_timeout(aresponses: ResponsesMockServer) -> None:
    """Test request timeout is handled correctly."""

    # Faking a timeout by sleeping
    async def reponse_handler(_: ClientResponse) -> Response:
        await asyncio.sleep(0.2)
        return aresponses.Response(body="Goodmorning!")

    aresponses.add("mijn.easyenergy.com", "/nl/api/tariff/test", "GET", reponse_handler)

    async with ClientSession() as session:
        client = EasyEnergy(session=session, request_timeout=0.1)
        with pytest.raises(EasyEnergyConnectionError):
            assert await client._request("test")


async def test_content_type(aresponses: ResponsesMockServer) -> None:
    """Test request content type error is handled correctly."""
    aresponses.add(
        "mijn.easyenergy.com",
        "/nl/api/tariff/test",
        "GET",
        aresponses.Response(
            status=200,
            headers={"Content-Type": "blabla/blabla"},
        ),
    )

    async with ClientSession() as session:
        client = EasyEnergy(
            session=session,
        )
        with pytest.raises(EasyEnergyError):
            assert await client._request("test")


async def test_client_error() -> None:
    """Test request client error is handled correctly."""
    async with ClientSession() as session:
        client = EasyEnergy(session=session)
        with (
            patch.object(
                session,
                "request",
                side_effect=ClientError,
            ),
            pytest.raises(EasyEnergyConnectionError),
        ):
            assert await client._request("test")


async def test_dns_error() -> None:
    """Test request DNS error is handled correctly."""
    async with ClientSession() as session:
        client = EasyEnergy(session=session)
        with (
            patch.object(
                DNSResolver,
                "query",
                side_effect=DNSError,
            ),
            pytest.raises(EasyEnergyConnectionError),
        ):
            assert await client._request("test")


async def test_empty_dns_error() -> None:
    """Test request DNS error is handled correctly."""
    async with ClientSession() as session:
        client = EasyEnergy(session=session)
        dns_result: Any = asyncio.Future()
        dns_result.set_result(None)
        with (
            patch.object(
                DNSResolver,
                "query",
                return_value=dns_result,
            ),
            pytest.raises(EasyEnergyConnectionError),
        ):
            assert await client._request("test")



"""Asynchronous Python client for the easyEnergy API."""

import asyncio
from datetime import date

from easyenergy import EasyEnergy, VatOption


async def main() -> None:
    """Show example on fetching the timestamp lists from easyEnergy."""
    async with EasyEnergy(vat=VatOption.INCLUDE) as client:
        today = date(2024, 1, 30)
        energy = await client.energy_prices(start_date=today, end_date=today)
        gas = await client.gas_prices(start_date=today, end_date=today)

        print("--- ENERGY / Usage ---")
        print(energy.timestamp_usage_prices)
        print()

        print("--- ENERGY / Return ---")
        print(energy.timestamp_return_prices)
        print()

        print("--- GAS ---")
        print(gas.timestamp_prices)


if __name__ == "__main__":
    asyncio.run(main())



"""Test the models."""

from datetime import UTC, date, datetime

import pytest
from aiohttp import ClientSession
from aresponses import ResponsesMockServer

from easyenergy import EasyEnergy, EasyEnergyNoDataError, Electricity, Gas, VatOption

from . import load_fixtures


@pytest.mark.freeze_time("2022-12-29 15:00:00+01:00")
async def test_electricity_model_usage(aresponses: ResponsesMockServer) -> None:
    """Test the electricity model for usage at 15:00:00 CET."""
    aresponses.add(
        "mijn.easyenergy.com",
        "/nl/api/tariff/getapxtariffs",
        "GET",
        aresponses.Response(
            status=200,
            headers={"Content-Type": "application/json"},
            text=load_fixtures("energy.json"),
        ),
    )
    async with ClientSession() as session:
        today = date(2022, 12, 29)
        client = EasyEnergy(session=session)
        energy: Electricity = await client.energy_prices(
            start_date=today,
            end_date=today,
            vat=VatOption.INCLUDE,
        )
        assert energy is not None
        assert isinstance(energy, Electricity)
        assert energy.extreme_usage_prices[1] == 0.13345
        assert energy.extreme_usage_prices[0] == -0.00277
        assert energy.average_usage_price == 0.06941
        assert energy.current_usage_price == 0.1199
        # The next hour price
        next_hour = datetime(2022, 12, 29, 15, 0, tzinfo=UTC)
        assert energy.price_at_time(next_hour) == 0.11979
        assert energy.lowest_usage_price_time == datetime.strptime(
            "2022-12-29 02:00",
            "%Y-%m-%d %H:%M",
        ).replace(tzinfo=UTC)
        assert energy.highest_usage_price_time == datetime.strptime(
            "2022-12-29 17:00",
            "%Y-%m-%d %H:%M",
        ).replace(tzinfo=UTC)
        assert energy.pct_of_max_usage == 89.85
        assert isinstance(energy.timestamp_usage_prices, list)
        assert energy.hours_priced_equal_or_lower_usage == 21
        assert energy.hours_priced_equal_or_higher_return == 5


@pytest.mark.freeze_time("2022-12-29 15:00:00+01:00")
async def test_electricity_model_return(aresponses: ResponsesMockServer) -> None:
    """Test the electricity model for return at 15:00:00 CET."""
    aresponses.add(
        "mijn.easyenergy.com",
        "/nl/api/tariff/getapxtariffs",
        "GET",
        aresponses.Response(
            status=200,
            headers={"Content-Type": "application/json"},
            text=load_fixtures("energy.json"),
        ),
    )
    async with ClientSession() as session:
        today = date(2022, 12, 29)
        client = EasyEnergy(session=session)
        energy: Electricity = await client.energy_prices(
            start_date=today,
            end_date=today,
            vat=VatOption.INCLUDE,
        )
        assert energy is not None
        assert isinstance(energy, Electricity)
        assert energy.extreme_return_prices[1] == 0.12243
        assert energy.extreme_return_prices[0] == -0.00254
        assert energy.average_return_price == 0.06368
        assert energy.current_return_price == 0.11
        # The next hour price
        next_hour = datetime(2022, 12, 29, 15, 0, tzinfo=UTC)
        assert energy.price_at_time(next_hour, data_type="return") == 0.1099
        assert energy.lowest_return_price_time == datetime.strptime(
            "2022-12-29 02:00",
            "%Y-%m-%d %H:%M",
        ).replace(tzinfo=UTC)
        assert energy.highest_return_price_time == datetime.strptime(
            "2022-12-29 17:00",
            "%Y-%m-%d %H:%M",
        ).replace(tzinfo=UTC)
        assert energy.pct_of_max_return == 89.85
        assert isinstance(energy.timestamp_return_prices, list)


@pytest.mark.freeze_time("2022-12-29 00:30:00+02:00")
async def test_electricity_midnight(aresponses: ResponsesMockServer) -> None:
    """Test the electricity model between 00:00 and 01:00 with in CEST."""
    aresponses.add(
        "mijn.easyenergy.com",
        "/nl/api/tariff/getapxtariffs",
        "GET",
        aresponses.Response(
            status=200,
            headers={"Content-Type": "application/json"},
            text=load_fixtures("energy.json"),
        ),
    )
    async with ClientSession() as session:
        today = date(2022, 12, 29)
        client = EasyEnergy(session=session)
        energy: Electricity = await client.energy_prices(
            start_date=today,
            end_date=today,
        )
        assert energy is not None
        assert energy.current_usage_price == 0.02341
        assert energy.current_return_price == 0.02148


async def test_electricity_none_data(aresponses: ResponsesMockServer) -> None:
    """Test when there is no data for the current datetime."""
    aresponses.add(
        "mijn.easyenergy.com",
        "/nl/api/tariff/getapxtariffs",
        "GET",
        aresponses.Response(
            status=200,
            headers={"Content-Type": "application/json"},
            text=load_fixtures("energy.json"),
        ),
    )
    async with ClientSession() as session:
        today = date(2022, 12, 29)
        client = EasyEnergy(session=session)
        energy: Electricity = await client.energy_prices(
            start_date=today,
            end_date=today,
        )
        assert energy is not None
        assert isinstance(energy, Electricity)
        assert energy.current_return_price is None
        assert energy.average_return_price == 0.06368


async def test_no_electricity_data(aresponses: ResponsesMockServer) -> None:
    """Test when there is no electricity data."""
    aresponses.add(
        "mijn.easyenergy.com",
        "/nl/api/tariff/getapxtariffs",
        "GET",
        aresponses.Response(
            status=200,
            headers={"Content-Type": "application/json"},
            text="[]",
        ),
    )
    async with ClientSession() as session:
        today = date(2022, 12, 16)
        client = EasyEnergy(session=session)
        with pytest.raises(EasyEnergyNoDataError):
            await client.energy_prices(start_date=today, end_date=today)


@pytest.mark.freeze_time("2022-12-14 15:00:00+01:00")
async def test_gas_model(aresponses: ResponsesMockServer) -> None:
    """Test the gas model - easyEnergy at 15:00:00 CET."""
    aresponses.add(
        "mijn.easyenergy.com",
        "/nl/api/tariff/getlebatariffs",
        "GET",
        aresponses.Response(
            status=200,
            headers={"Content-Type": "application/json"},
            text=load_fixtures("gas.json"),
        ),
    )
    async with ClientSession() as session:
        today = date(2022, 12, 14)
        client = EasyEnergy(session=session)
        gas: Gas = await client.gas_prices(
            start_date=today,
            end_date=today,
            vat=VatOption.INCLUDE,
        )
        assert gas is not None
        assert isinstance(gas, Gas)
        assert gas.extreme_prices[1] == 1.48534
        assert gas.extreme_prices[0] == 1.4645
        assert gas.average_price == 1.47951
        assert gas.current_price == 1.48534
        assert isinstance(gas.timestamp_prices, list)


@pytest.mark.freeze_time("2022-12-14 04:00:00+01:00")
async def test_gas_morning_model(aresponses: ResponsesMockServer) -> None:
    """Test the gas model in the morning - easyEnergy at 04:00:00 CET."""
    aresponses.add(
        "mijn.easyenergy.com",
        "/nl/api/tariff/getlebatariffs",
        "GET",
        aresponses.Response(
            status=200,
            headers={"Content-Type": "application/json"},
            text=load_fixtures("gas.json"),
        ),
    )
    async with ClientSession() as session:
        today = date(2022, 12, 14)
        client = EasyEnergy(session=session)
        gas: Gas = await client.gas_prices(
            start_date=today,
            end_date=today,
            vat=VatOption.INCLUDE,
        )
        assert gas is not None
        assert isinstance(gas, Gas)
        assert gas.extreme_prices[1] == 1.48534
        assert gas.extreme_prices[0] == 1.4645
        assert gas.average_price == 1.47951
        assert gas.current_price == 1.4645


async def test_gas_none_data(aresponses: ResponsesMockServer) -> None:
    """Test when there is no data for the current datetime."""
    aresponses.add(
        "mijn.easyenergy.com",
        "/nl/api/tariff/getlebatariffs",
        "GET",
        aresponses.Response(
            status=200,
            headers={"Content-Type": "application/json"},
            text=load_fixtures("gas.json"),
        ),
    )
    async with ClientSession() as session:
        today = date(2022, 12, 14)
        client = EasyEnergy(session=session)
        gas: Gas = await client.gas_prices(start_date=today, end_date=today)
        assert gas is not None
        assert isinstance(gas, Gas)
        assert gas.current_price is None
        assert gas.average_price == 1.47951


async def test_no_gas_data(aresponses: ResponsesMockServer) -> None:
    """Test when there is no gas data."""
    aresponses.add(
        "mijn.easyenergy.com",
        "/nl/api/tariff/getlebatariffs",
        "GET",
        aresponses.Response(
            status=200,
            headers={"Content-Type": "application/json"},
            text="[]",
        ),
    )
    async with ClientSession() as session:
        today = date(2022, 12, 16)
        client = EasyEnergy(session=session)
        with pytest.raises(EasyEnergyNoDataError):
            await client.gas_prices(start_date=today, end_date=today)
